# -*- coding: utf-8 -*-
"""
Serbian (sr/rs) transliteration mappings.

ISO 639-1 language code: sr
ISO 3166-1 country code alias: rs
"""

# This dictionary is to transliterate from Serbian Cyrillic to Latin.
SR_CYR_TO_LAT_DICT = {
    u'А': u'A', u'а': u'a',
    u'Б': u'B', u'б': u'b',
    u'В': u'V', u'в': u'v',
    u'Г': u'G', u'г': u'g',
    u'Д': u'D', u'д': u'd',
    u'Ђ': u'Đ', u'ђ': u'đ',
    u'Е': u'E', u'е': u'e',
    u'Ж': u'Ž', u'ж': u'ž',
    u'З': u'Z', u'з': u'z',
    u'И': u'I', u'и': u'i',
    u'Ј': u'J', u'ј': u'j',
    u'К': u'K', u'к': u'k',
    u'Л': u'L', u'л': u'l',
    u'Љ': u'Lj', u'љ': u'lj',
    u'М': u'M', u'м': u'm',
    u'Н': u'N', u'н': u'n',
    u'Њ': u'Nj', u'њ': u'nj',
    u'О': u'O', u'о': u'o',
    u'П': u'P', u'п': u'p',
    u'Р': u'R', u'р': u'r',
    u'С': u'S', u'с': u's',
    u'Т': u'T', u'т': u't',
    u'Ћ': u'Ć', u'ћ': u'ć',
    u'У': u'U', u'у': u'u',
    u'Ф': u'F', u'ф': u'f',
    u'Х': u'H', u'х': u'h',
    u'Ц': u'C', u'ц': u'c',
    u'Ч': u'Č', u'ч': u'č',
    u'Џ': u'Dž', u'џ': u'dž',
    u'Ш': u'Š', u'ш': u'š',
}

# This dictionary is to transliterate from Serbian Latin to Cyrillic.
# Let's build it by simply swapping keys and values of previous dictionary.
SR_LAT_TO_CYR_DICT = {y: x for x, y in iter(SR_CYR_TO_LAT_DICT.items())}



# -*- coding: utf-8 -*-
"""
Bulgarian (bg) transliteration mappings.

ISO 639-1 language code: bg

Supports accented I with grave for stress marking and homograph disambiguation.
Following ISO 9:1995.
"""

import copy
from .ru import RU_CYR_TO_LAT_DICT

# Transliterate from Bulgarian Cyrillic to Latin
BG_CYR_TO_LAT_DICT = copy.deepcopy(RU_CYR_TO_LAT_DICT)

# There are a couple of letters that don't exist in Bulgarian:
del BG_CYR_TO_LAT_DICT[u"Ё"]
del BG_CYR_TO_LAT_DICT[u"ё"]
del BG_CYR_TO_LAT_DICT[u"Ы"]
del BG_CYR_TO_LAT_DICT[u"ы"]
del BG_CYR_TO_LAT_DICT[u"Э"]
del BG_CYR_TO_LAT_DICT[u"э"]

# Some letters that are pronounced differently
BG_CYR_TO_LAT_DICT[u"Й"] = u"Y"
BG_CYR_TO_LAT_DICT[u"й"] = u"y"
BG_CYR_TO_LAT_DICT[u"Х"] = u"H"
BG_CYR_TO_LAT_DICT[u"х"] = u"h"
BG_CYR_TO_LAT_DICT[u"Ц"] = u"TS"
BG_CYR_TO_LAT_DICT[u"ц"] = u"ts"
BG_CYR_TO_LAT_DICT[u"Щ"] = u"SHT"
BG_CYR_TO_LAT_DICT[u"щ"] = u"sht"
BG_CYR_TO_LAT_DICT[u"Ю"] = u"YU"
BG_CYR_TO_LAT_DICT[u"ю"] = u"yu"
BG_CYR_TO_LAT_DICT[u"Я"] = u"YA"
BG_CYR_TO_LAT_DICT[u"я"] = u"ya"

# The following letters use the pre-2012 "Andreichin" system for lettering,
# because in the newest "Ivanov" system "a" and "y" translate to two Bulgarian
# letters and choosing to which one depends on the word and text context
# https://en.wikipedia.org/wiki/Romanization_of_Bulgarian
BG_CYR_TO_LAT_DICT[u"Ъ"] = u"Ă"
BG_CYR_TO_LAT_DICT[u"ъ"] = u"ă"
BG_CYR_TO_LAT_DICT[u"Ь"] = u"J"
BG_CYR_TO_LAT_DICT[u"ь"] = u"j"

# Transliterate from Latin Bulgarian to Cyrillic.
# Build this BEFORE adding accented Cyrillic characters to avoid reverse mapping conflicts
BG_LAT_TO_CYR_DICT = {y: x for x, y in iter(BG_CYR_TO_LAT_DICT.items())}

# Accented vowels with grave accent (used for stress marking and homograph disambiguation)
# Following ISO 9:1995
# Source: https://en.wikipedia.org/wiki/I_with_grave_(Cyrillic)
# Used to distinguish: ѝ (her) vs и (and)
#
# By default (preserve_accents=False), accented Cyrillic maps to unaccented Latin
BG_CYR_TO_LAT_DICT[u"Ѝ"] = u"I"  # Cyrillic I with grave → I (U+040D)
BG_CYR_TO_LAT_DICT[u"ѝ"] = u"i"  # Cyrillic i with grave → i (U+045D)

# Accented map: When preserve_accents=True, these override the standard mappings
BG_CYR_TO_LAT_ACCENTED_DICT = {
    u"Ѝ": u"Ì",  # Cyrillic I with grave → Ì
    u"ѝ": u"ì",  # Cyrillic i with grave → ì
}

BG_LAT_TO_CYR_DICT.update({
    u"ZH": u"Ж", u"Zh": u"Ж", u"zh": u"ж",
    u"TS": u"Ц", u"Ts": u"Ц", u"ts": u"ц",
    u"CH": u"Ч", u"Ch": u"Ч", u"ch": u"ч",
    u"SH": u"Ш", u"Sh": u"Ш", u"sh": u"ш",
    u"SHT": u"Щ", u"Sht": u"Щ", u"sht": u"щ",
    u"YU": u"Ю", u"Yu": u"Ю", u"yu": u"ю",
    u"YA": u"Я", u"Ya": u"Я", u"ya": u"я",
    # Accented Latin to unaccented Cyrillic (preserve_accents=False)
    u"Ì": u"И", u"ì": u"и",  # Latin I with grave → Cyrillic I
})

# Accented map for Latin→Cyrillic: When preserve_accents=True, these override
BG_LAT_TO_CYR_ACCENTED_DICT = {
    u"Ì": u"Ѝ", u"ì": u"ѝ",  # Latin I with grave → Cyrillic Ѝ
}



# -*- coding: utf-8 -*-
"""
Russian (ru) transliteration mappings.

ISO 639-1 language code: ru

Transliteration follows GOST 7.79-2000 System B.
"""

# This dictionary is to transliterate from Russian Cyrillic to Latin (GOST_7.79-2000 System B).
RU_CYR_TO_LAT_DICT = {
    u"А": u"A", u"а": u"a",
    u"Б": u"B", u"б": u"b",
    u"В": u"V", u"в": u"v",
    u"Г": u"G", u"г": u"g",
    u"Д": u"D", u"д": u"d",
    u"Е": u"E", u"е": u"e",
    u"Ё": u"YO", u"ё": u"yo",
    u"Ж": u"ZH", u"ж": u"zh",
    u"З": u"Z", u"з": u"z",
    u"И": u"I", u"и": u"i",
    u"Й": u"J", u"й": u"j",
    u"К": u"K", u"к": u"k",
    u"Л": u"L", u"л": u"l",
    u"М": u"M", u"м": u"m",
    u"Н": u"N", u"н": u"n",
    u"О": u"O", u"о": u"o",
    u"П": u"P", u"п": u"p",
    u"Р": u"R", u"р": u"r",
    u"С": u"S", u"с": u"s",
    u"Т": u"T", u"т": u"t",
    u"У": u"U", u"у": u"u",
    u"Ф": u"F", u"ф": u"f",
    u"Х": u"H", u"х": u"h",
    u"Ц": u"CZ", u"ц": u"cz",
    u"Ч": u"CH", u"ч": u"ch",
    u"Ш": u"SH", u"ш": u"sh",
    u"Щ": u"SHH", u"щ": u"shh",
    u"Ъ": u"''", u"ъ": u"''",
    u"Ы": u"Y'", u"ы": u"y'",
    u"Ь": u"'", u"ь": u"'",
    u"Э": u"E'", u"э": u"e'",
    u"Ю": u"Yu", u"ю": u"yu",
    u"Я": u"Ya", u"я": u"ya",
}

# This dictionary is to transliterate from Russian Latin to Cyrillic.
RU_LAT_TO_CYR_DICT = {y: x for x, y in RU_CYR_TO_LAT_DICT.items()}
RU_LAT_TO_CYR_DICT.update({
    u"''": u"ъ",
    u"'": u"ь",
    u"C": u"К", u"c": u"к",
    u"CK": u"К", u"Ck": u"К", u"ck": u"к",
    u"JA": u"ЖА", u"Ja": u"Жа", u"ja": u"жа",
    u"JE": u"ЖЕ", u"Je": u"Же", u"je": u"же",
    u"JI": u"ЖИ", u"Ji": u"Жи", u"ji": u"жи",
    u"JO": u"ЖО", u"Jo": u"Жо", u"jo": u"жо",
    u"JU": u"ЖУ", u"Ju": u"Жу", u"ju": u"жу",
    u"PH": u"Ф", u"Ph": u"Ф", u"ph": u"ф",
    u"TH": u"З", u"Th": u"З", u"th": u"з",
    u"W": u"В", u"w": u"в", u"Q": u"К", u"q": u"к",
    u"WH": u"В", u"Wh": u"В", u"wh": u"в",
    u"Y": u"И", u"y": u"и",
    u"YA": u"Я", u"Ya": u"Я", u"ya": u"я",
    u"YE": u"Е", u"Ye": u"Е", u"ye": u"е",
    u"YI": u"И", u"Yi": u"И", u"yi": u"и",
    u"YO": u"Ё", u"Yo": u"Ё", u"yo": u"ё",
    u"YU": u"Ю", u"Yu": u"Ю", u"yu": u"ю",
    u"Y'": u"ы", u"y'": u"ы",
    u"iy": u"ый", u"ij": u"ый",  # dobriy => добрый
})



# -*- coding: utf-8 -*-
"""
Mongolian (mn) transliteration mappings.

ISO 639-1 language code: mn

This version of Mongolian Latin <-> Cyrillic is based on MNS 5217:2012
as far as I know this is the latest standard. Inform me @ https://github.com/Serbipunk

References:
https://gogo.mn/r/101115
https://en.wikipedia.org/wiki/Mongolian_Cyrillic_alphabet
"""

# This list contains alternating Cyrillic and Latin mappings
# Format: [Cyrillic_upper, Latin_upper, Cyrillic_lower, Latin_lower, ...]
MN_CYR_LAT_LIST = [
    u"А", u"A", u"а", u"a",
    u"Э", u"E", u"э", u"e",
    u"И", u"I", u"и", u"i",  # i
    u"О", u"O", u"о", u"o",
    u"У", u"U", u"у", u"u",
    u"Ө", u"Ö", u"ө", u"ö",
    u"Ү", u"Ü", u"ү", u"ü",
    u"Н", u"N", u"н", u"n",
    u"М", u"M", u"м", u"m",
    u"Л", u"L", u"л", u"l",
    u"В", u"V", u"в", u"v",
    u"П", u"P", u"п", u"p",
    u"Ф", u"F", u"ф", u"f",
    u"К", u"K", u"к", u"k",
    u"Х", u"Kh", u"х", u"kh",        # lat 1
    u"Х", u"KH", u"х", u"kH",        # lat 1
    u"Г", u"G", u"г", u"g",
    u"С", u"S", u"с", u"s",
    u"Ш", u"Sh", u"ш", u"sh",  # sh  # lat2
    u"Ш", u"SH", u"ш", u"sH",  # sh  # lat2
    u"Т", u"T", u"т", u"t",
    u"Д", u"D", u"д", u"d",
    u"Ц", u"Ts", u"ц", u"ts",        # lat3
    u"Ц", u"TS", u"ц", u"tS",        # lat3
    u"Ч", u"Ch", u"ч", u"ch",        # lat4
    u"Ч", u"CH", u"ч", u"cH",        # lat4
    u"З", u"Z", u"з", u"z",
    u"Ж", u"J", u"ж", u"j",
    u"Й", u"I", u"й", u"i",  # i * 2
    u"Р", u"R", u"р", u"r",
    u"Б", u"B", u"б", u"b",
    u"Е", u"Ye", u"е", u"ye",             # lat 5
    u"Е", u"YE", u"е", u"yE",             # lat 5
    u"Ё", u"Yo", u"ё", u"yo",             # lat 6
    u"Ё", u"YO", u"ё", u"yO",             # lat 6
    u"Ъ", u"I", u"ъ", u"i",  # i * 3
    u"Ы", u"Y", u"ы", u"y",
    u"Ь", u"I", u"ь", u"i",  # i * 4
    u"Ю", u"Yu", u"ю", u"yu",             # lat 8
    u"Ю", u"YU", u"ю", u"yU",             # lat 8
    u"Я", u"Ya", u"я", u"ya",             # lat 9
    u"Я", u"YA", u"я", u"yA",             # lat 9
]
# Building the dictionary with the filter to skip pairs with 2-character Latin letters where the second character is uppercase
MN_CYR_TO_LAT_DICT = {
    c: l for c, l in zip(MN_CYR_LAT_LIST[::2], MN_CYR_LAT_LIST[1::2])
    if not (len(l) == 2 and l[1].isupper())
}

# Handle Щ (shcha): This letter is part of Mongolian Cyrillic (inherited from Russian)
# but is rarely used in practice. It's pronounced the same as Ш (/ʃ/), so both
# transliterate to "Sh/sh" in Latin. When going Latin → Cyrillic, "Sh" defaults to Ш.
MN_CYR_TO_LAT_DICT['Щ'] = 'Sh'
MN_CYR_TO_LAT_DICT['щ'] = 'sh'

MN_LAT_TO_CYR_DICT = dict([(l, c) for c, l in zip(MN_CYR_LAT_LIST[-2::-2], MN_CYR_LAT_LIST[-1::-2])])



import cyrtranslit
from cyrtranslit.mapping import TRANSLIT_DICT
from argparse import ArgumentParser, FileType
import os
import sys

def __is_valid_language_code(parse, arg):
    ''' Validates inputted two-letter language code.
    :param parse: The argument parser. Used to display error message.
    :param arg: The language code argument.
    '''
    if arg.lower() not in TRANSLIT_DICT:
        parser.error("The language code %s is not supported. Support language codes are: %s." % (arg, ", ".join(TRANSLIT_DICT.keys()).upper()))
    else:
        return arg

def main():
    # Setup argument parser
    parser = ArgumentParser(description="Transiliterate text in a given file.")

    # Input file.
    # Not required.
    parser.add_argument("-i", dest="input_file", required=False,
                        help="input file",
                        default=None)

    # Output file.
    # Not required. If not specified, transliteration will appear as console output.
    parser.add_argument("-o", dest="output_file", required=False,
                        help="ouput file",
                        default=None)

    # Language code for cyrillic text in inputted file.
    # Required.
    parser.add_argument("-l", dest="language_code", required=True,
                        help="two-letter ISO 639-1 language code of cyrillic text",
                        type=lambda x: __is_valid_language_code(parser, x))

    # Flag for reverse transliteration, i.e. from latin/roman alphabet to cyrillic.
    parser.add_argument("-c", dest="to_cyrillic", action='store_true',
                        help="Parse latin characters to cyrillic (reverse of transliteration)")

    # Flag to preserve accent marks in transliteration
    parser.add_argument("-p", "--preserve-accents", dest="preserve_accents", action='store_true',
                        help="Preserve accent marks (e.g., Macedonian/Bulgarian Ѐ→È, ѝ→ì instead of Ѐ→E, ѝ→i)")

    # Input file encoding.
    # Not required. Defaults to utf-8 with fallback to common Cyrillic encodings.
    parser.add_argument("-e", "--encoding", dest="encoding", required=False,
                        help="input file encoding (default: utf-8 with automatic fallback to windows-1251, iso-8859-5, koi8-r, cp866)",
                        default="utf-8")

    # Parse arguments
    args = parser.parse_args()

    # Fetch arguments.
    lang_code = args.language_code
    to_cyrillic = args.to_cyrillic
    preserve_accents = args.preserve_accents
    encoding = args.encoding

    # Open input file with proper encoding handling
    if args.input_file:
        # Try specified encoding first
        file_input = None
        tried_encodings = [encoding]

        # Helper function to test if an encoding works
        def try_encoding(filepath, enc):
            try:
                f = open(filepath, 'r', encoding=enc)
                # Try to read the file to actually test the encoding
                f.read()
                # If successful, reopen from the beginning
                f.close()
                return open(filepath, 'r', encoding=enc)
            except (UnicodeDecodeError, LookupError):
                if f:
                    f.close()
                return None

        file_input = try_encoding(args.input_file, encoding)

        if file_input is None:
            # If specified encoding fails, try common Cyrillic encodings as fallback
            fallback_encodings = ['windows-1251', 'iso-8859-5', 'koi8-r', 'cp866']

            for fallback_enc in fallback_encodings:
                if fallback_enc == encoding:
                    continue  # Already tried this one
                tried_encodings.append(fallback_enc)
                file_input = try_encoding(args.input_file, fallback_enc)
                if file_input is not None:
                    print(f"Warning: Failed to decode with {encoding}, using {fallback_enc} instead.", file=sys.stderr)
                    break

            if file_input is None:
                print(f"Error: Unable to decode file with any of the attempted encodings: {', '.join(tried_encodings)}", file=sys.stderr)
                print(f"Try specifying the correct encoding with -e/--encoding parameter.", file=sys.stderr)
                print(f"Common Cyrillic encodings: windows-1251, iso-8859-5, koi8-r, cp866", file=sys.stderr)
                sys.exit(1)
    else:
        file_input = sys.stdin

    # Open output file
    if args.output_file:
        file_output = open(args.output_file, 'w', encoding='utf-8')
    else:
        file_output = sys.stdout

    # Transliterate and write directly to output line by line
    try:
        for line in file_input:
            if to_cyrillic is True:
                file_output.write(cyrtranslit.to_cyrillic(line, lang_code=lang_code, preserve_accents=preserve_accents))
            else:
                file_output.write(cyrtranslit.to_latin(line, lang_code=lang_code, preserve_accents=preserve_accents))
    finally:
        # Close streams if they're not stdin/stdout
        if args.input_file and file_input:
            file_input.close()
        if args.output_file and file_output:
            file_output.close()

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Cyrillic transliteration mapping package.

This package contains transliteration mappings for various Cyrillic and Greek scripts.
Each language has its own module with specific transliteration dictionaries.

Supported languages:
- bg: Bulgarian
- by: Belarusian
- el: Greek
- me: Montenegrin
- mk: Macedonian
- mn: Mongolian
- rs: Serbian (ISO 3166-1 country code alias)
- ru: Russian
- sr: Serbian
- tj: Tajik
- ua: Ukrainian
"""

# Import all language-specific mappings
from .sr import SR_CYR_TO_LAT_DICT, SR_LAT_TO_CYR_DICT
from .me import ME_CYR_TO_LAT_DICT, ME_LAT_TO_CYR_DICT
from .mk import (
    MK_CYR_TO_LAT_DICT,
    MK_LAT_TO_CYR_DICT,
    MK_CYR_TO_LAT_ACCENTED_DICT,
    MK_LAT_TO_CYR_ACCENTED_DICT
)
from .ru import RU_CYR_TO_LAT_DICT, RU_LAT_TO_CYR_DICT
from .tj import TJ_CYR_TO_LAT_DICT, TJ_LAT_TO_CYR_DICT
from .bg import (
    BG_CYR_TO_LAT_DICT,
    BG_LAT_TO_CYR_DICT,
    BG_CYR_TO_LAT_ACCENTED_DICT,
    BG_LAT_TO_CYR_ACCENTED_DICT
)
from .ua import UA_CYR_TO_LAT_DICT, UA_LAT_TO_CYR_DICT
from .by import BY_CYR_TO_LAT_DICT, BY_LAT_TO_CYR_DICT
from .mn import MN_CYR_TO_LAT_DICT, MN_LAT_TO_CYR_DICT
from .el import EL_GRE_TO_LAT_DICT, EL_LAT_TO_GRE_DICT

# Bundle up all the dictionaries in a lookup dictionary
TRANSLIT_DICT = {
    'sr': {  # Serbian (ISO 639-1 language code)
        'tolatin': SR_CYR_TO_LAT_DICT,
        'tocyrillic': SR_LAT_TO_CYR_DICT
    },
    'rs': {  # Serbian (ISO 3166-1 country code alias)
        'tolatin': SR_CYR_TO_LAT_DICT,
        'tocyrillic': SR_LAT_TO_CYR_DICT
    },
    'me': {  # Montenegro
        'tolatin': ME_CYR_TO_LAT_DICT,
        'tocyrillic': ME_LAT_TO_CYR_DICT
    },
    'mk': {  # Macedonia
        'tolatin': MK_CYR_TO_LAT_DICT,
        'tocyrillic': MK_LAT_TO_CYR_DICT,
        'tolatin_accented': MK_CYR_TO_LAT_ACCENTED_DICT,
        'tocyrillic_accented': MK_LAT_TO_CYR_ACCENTED_DICT
    },
    'ru': {  # Russian
        'tolatin': RU_CYR_TO_LAT_DICT,
        'tocyrillic': RU_LAT_TO_CYR_DICT
    },
    'tj': {  # Tajik
        'tolatin': TJ_CYR_TO_LAT_DICT,
        'tocyrillic': TJ_LAT_TO_CYR_DICT
    },
    'bg': {  # Bulgarian
        'tolatin': BG_CYR_TO_LAT_DICT,
        'tocyrillic': BG_LAT_TO_CYR_DICT,
        'tolatin_accented': BG_CYR_TO_LAT_ACCENTED_DICT,
        'tocyrillic_accented': BG_LAT_TO_CYR_ACCENTED_DICT
    },
    'ua': {  # Ukrainian
        'tolatin': UA_CYR_TO_LAT_DICT,
        'tocyrillic': UA_LAT_TO_CYR_DICT
    },
    'by': {  # Belarusian
        'tolatin': BY_CYR_TO_LAT_DICT,
        'tocyrillic': BY_LAT_TO_CYR_DICT
    },
    'mn': {  # Mongolian
        'tolatin': MN_CYR_TO_LAT_DICT,
        'tocyrillic': MN_LAT_TO_CYR_DICT
    },
    'el': {  # Greek (ISO 639-1 language code)
        'tolatin': EL_GRE_TO_LAT_DICT,
        'tocyrillic': EL_LAT_TO_GRE_DICT
    }
}

# Export the main dictionary for backward compatibility
__all__ = ['TRANSLIT_DICT']



# -*- coding: utf-8 -*-
"""
Tajik (tj) transliteration mappings.

ISO 639-1 language code: tj

Transliteration follows ISO 9 (1995).
https://en.wikipedia.org/wiki/Tajik_alphabet#Cyrillic
"""

import copy
from .ru import RU_CYR_TO_LAT_DICT

# Transliterate from Tajik cyrillic to latin
TJ_CYR_TO_LAT_DICT = copy.deepcopy(RU_CYR_TO_LAT_DICT)
# Change Mapping according to ISO 9 (1995)
TJ_CYR_TO_LAT_DICT[u"Э"] = u"È"
TJ_CYR_TO_LAT_DICT[u"э"] = u"è"
TJ_CYR_TO_LAT_DICT[u"ъ"] = u"’"
TJ_CYR_TO_LAT_DICT[u"Х"] = u"H"
TJ_CYR_TO_LAT_DICT[u"х"] = u"h"
TJ_CYR_TO_LAT_DICT[u"Ч"] = u"Č"
TJ_CYR_TO_LAT_DICT[u"ч"] = u"č"
TJ_CYR_TO_LAT_DICT[u"Ж"] = u"Ž"
TJ_CYR_TO_LAT_DICT[u"ж"] = u"ž"
TJ_CYR_TO_LAT_DICT[u"Ё"] = u"Ë"
TJ_CYR_TO_LAT_DICT[u"ё"] = u"ë"
TJ_CYR_TO_LAT_DICT[u"Ш"] = u"Š"
TJ_CYR_TO_LAT_DICT[u"ш"] = u"š"
TJ_CYR_TO_LAT_DICT[u"Ю"] = u"Û"
TJ_CYR_TO_LAT_DICT[u"ю"] = u"û"
TJ_CYR_TO_LAT_DICT[u"Я"] = u"Â"
TJ_CYR_TO_LAT_DICT[u"я"] = u"â"
# delete letters not used
del TJ_CYR_TO_LAT_DICT[u"Ц"]
del TJ_CYR_TO_LAT_DICT[u"ц"]
del TJ_CYR_TO_LAT_DICT[u"Щ"]
del TJ_CYR_TO_LAT_DICT[u"щ"]
del TJ_CYR_TO_LAT_DICT[u"Ы"]
del TJ_CYR_TO_LAT_DICT[u"ы"]

# update the dict for the additional letters in the tajik cyrillic alphabet ( Ғ, Ӣ, Қ, Ӯ, Ҳ, Ҷ )
# see https://en.wikipedia.org/wiki/Tajik_alphabet#Cyrillic
TJ_CYR_TO_LAT_DICT.update({
    u"Ғ": u"Ǧ", u"ғ": u"ǧ",
    u"Ӣ": u"Ī", u"ӣ": u"ī",
    u"Қ": u"Q", u"қ": u"q",
    u"Ӯ": u"Ū", u"ӯ": u"ū",
    u"Ҳ": u"Ḩ", u"ҳ": u"ḩ",
    u"Ҷ": u"Ç", u"ҷ": u"ç"
})

# transliterate from latin tajik to cyrillic
TJ_LAT_TO_CYR_DICT = {y: x for x, y in iter(TJ_CYR_TO_LAT_DICT.items())}



# -*- coding: utf-8 -*-
"""
Macedonian (mk) transliteration mappings.

ISO 639-1 language code: mk

Supports accented vowels with grave accent used for homograph disambiguation:
- Ѐ/ѐ (U+0400/U+0450) - IE with grave
- Ѝ/ѝ (U+040D/U+045D) - I with grave

Following ISO 9:1968/1995, adopted by Macedonian Academy of Arts and Sciences (1970).
"""

import copy
from .sr import SR_CYR_TO_LAT_DICT

# Build the dictionaries to transliterate Macedonian Cyrillic to Latin and vice versa.
MK_CYR_TO_LAT_DICT = copy.deepcopy(SR_CYR_TO_LAT_DICT)

# Differences with Serbian:
# 1) Between Ze (З з) and I (И и) is the letter Dze (Ѕ ѕ), which looks like the Latin letter S and represents /d͡z/.
MK_CYR_TO_LAT_DICT[u'Ѕ'] = u'Dz'
MK_CYR_TO_LAT_DICT[u'ѕ'] = u'dz'

# 2) Dje (Ђ ђ) is replaced by Gje (Ѓ ѓ), which represents /ɟ/ (voiced palatal stop).
# In some dialects, it represents /d͡ʑ/ instead, like Dje
# It is written ⟨Ǵ ǵ⟩ in the corresponding Macedonian Latin alphabet.
del MK_CYR_TO_LAT_DICT[u'Ђ']
del MK_CYR_TO_LAT_DICT[u'ђ']
MK_CYR_TO_LAT_DICT[u'Ѓ'] = u'Ǵ'
MK_CYR_TO_LAT_DICT[u'ѓ'] = u'ǵ'

# 3) Tshe (Ћ ћ) is replaced by Kje (Ќ ќ), which represents /c/ (voiceless palatal stop).
# In some dialects, it represents /t͡ɕ/ instead, like Tshe.
# It is written ⟨Ḱ ḱ⟩ in the corresponding Macedonian Latin alphabet.
del MK_CYR_TO_LAT_DICT[u'Ћ']
del MK_CYR_TO_LAT_DICT[u'ћ']
MK_CYR_TO_LAT_DICT[u'Ќ'] = u'Ḱ'
MK_CYR_TO_LAT_DICT[u'ќ'] = u'ḱ'

# This dictionary is to transliterate from Macedonian Latin to Cyrillic.
# Build this BEFORE adding accented Cyrillic characters to avoid reverse mapping conflicts
MK_LAT_TO_CYR_DICT = {y: x for x, y in iter(MK_CYR_TO_LAT_DICT.items())}

# 4) Accented vowels with grave accent (used to disambiguate homographs in Macedonian)
# Following ISO 9:1968/1995, adopted by Macedonian Academy of Arts and Sciences in 1970
# Source: https://en.wikipedia.org/wiki/I_with_grave_(Cyrillic)
# These are used to distinguish homographs:
# - ѝ (her) vs и (and)
# - нѐ (us) vs не (no)
# - сѐ (everything) vs се (short reflexive pronoun)
#
# By default (preserve_accents=False), accented Cyrillic maps to unaccented Latin
MK_CYR_TO_LAT_DICT[u'Ѐ'] = u'E'  # Cyrillic E with grave → E (U+0400)
MK_CYR_TO_LAT_DICT[u'ѐ'] = u'e'  # Cyrillic e with grave → e (U+0450)
MK_CYR_TO_LAT_DICT[u'Ѝ'] = u'I'  # Cyrillic I with grave → I (U+040D)
MK_CYR_TO_LAT_DICT[u'ѝ'] = u'i'  # Cyrillic i with grave → i (U+045D)

# Accented map: When preserve_accents=True, these override the standard mappings
MK_CYR_TO_LAT_ACCENTED_DICT = {
    u'Ѐ': u'È',  # Cyrillic E with grave → È
    u'ѐ': u'è',  # Cyrillic e with grave → è
    u'Ѝ': u'Ì',  # Cyrillic I with grave → Ì
    u'ѝ': u'ì',  # Cyrillic i with grave → ì
}

# Add mappings for accented Latin to unaccented Cyrillic (preserve_accents=False)
MK_LAT_TO_CYR_DICT.update({
    u'È': u'Е', u'è': u'е',  # Latin E with grave → Cyrillic E
    u'Ì': u'И', u'ì': u'и',  # Latin I with grave → Cyrillic I
})

# Accented map for Latin→Cyrillic: When preserve_accents=True, these override
MK_LAT_TO_CYR_ACCENTED_DICT = {
    u'È': u'Ѐ', u'è': u'ѐ',  # Latin E with grave → Cyrillic Ѐ
    u'Ì': u'Ѝ', u'ì': u'ѝ',  # Latin I with grave → Cyrillic Ѝ
}



# -*- coding: utf-8 -*-
"""
Greek (el) transliteration mappings.

ISO 639-1 language code: el

Transliteration follows ELOT 743 / ISO 843.
https://en.wikipedia.org/wiki/Greek_alphabet
https://en.wikipedia.org/wiki/Romanization_of_Greek
"""

# Transliterate from Greek to Latin (ELOT 743 / ISO 843)
EL_GRE_TO_LAT_DICT = {
    u"Α": u"A", u"α": u"a", u"Ά": u"A", u"ά": u"a",  # alpha (with/without tonos)
    u"Β": u"V", u"β": u"v",
    u"Γ": u"G", u"γ": u"g",
    u"Δ": u"D", u"δ": u"d",
    u"Ε": u"E", u"ε": u"e", u"Έ": u"E", u"έ": u"e",  # epsilon (with/without tonos)
    u"Ζ": u"Z", u"ζ": u"z",
    u"Η": u"H", u"η": u"h", u"Ή": u"H", u"ή": u"h",  # eta (with/without tonos)
    u"Θ": u"Th", u"θ": u"th",
    u"Ι": u"I", u"ι": u"i", u"Ί": u"I", u"ί": u"i", u"Ϊ": u"I", u"ϊ": u"i",  # iota (with tonos/dialytika)
    u"Κ": u"K", u"κ": u"k",
    u"Λ": u"L", u"λ": u"l",
    u"Μ": u"M", u"μ": u"m",
    u"Ν": u"N", u"ν": u"n",
    u"Ξ": u"X", u"ξ": u"x",
    u"Ο": u"O", u"ο": u"o", u"Ό": u"O", u"ό": u"o",  # omicron (with/without tonos)
    u"Π": u"P", u"π": u"p",
    u"Ρ": u"R", u"ρ": u"r",
    u"Σ": u"S", u"σ": u"s", u"ς": u"s",  # sigma (ς is final sigma)
    u"Τ": u"T", u"τ": u"t",
    u"Υ": u"Y", u"υ": u"y", u"Ύ": u"Y", u"ύ": u"y", u"Ϋ": u"Y", u"ϋ": u"y",  # upsilon (with tonos/dialytika)
    u"Φ": u"F", u"φ": u"f",
    u"Χ": u"Ch", u"χ": u"ch",
    u"Ψ": u"Ps", u"ψ": u"ps",
    u"Ω": u"W", u"ω": u"w", u"Ώ": u"W", u"ώ": u"w",  # omega (with/without tonos)
}

# This dictionary is to transliterate from Latin to Greek
# Build the reverse mapping, but only include unaccented letters
# (accented vowels transliterate to same Latin as unaccented, so we default to unaccented)
EL_LAT_TO_GRE_DICT = {
    u"A": u"Α", u"a": u"α",
    u"V": u"Β", u"v": u"β",
    u"G": u"Γ", u"g": u"γ",
    u"D": u"Δ", u"d": u"δ",
    u"E": u"Ε", u"e": u"ε",
    u"Z": u"Ζ", u"z": u"ζ",
    u"H": u"Η", u"h": u"η",
    u"I": u"Ι", u"i": u"ι",
    u"K": u"Κ", u"k": u"κ",
    u"L": u"Λ", u"l": u"λ",
    u"M": u"Μ", u"m": u"μ",
    u"N": u"Ν", u"n": u"ν",
    u"X": u"Ξ", u"x": u"ξ",
    u"O": u"Ο", u"o": u"ο",
    u"P": u"Π", u"p": u"π",
    u"R": u"Ρ", u"r": u"ρ",
    u"S": u"Σ", u"s": u"σ",
    u"T": u"Τ", u"t": u"τ",
    u"Y": u"Υ", u"y": u"υ",
    u"F": u"Φ", u"f": u"φ",
    u"W": u"Ω", u"w": u"ω",
}
EL_LAT_TO_GRE_DICT.update({
    u"TH": u"Θ", u"Th": u"Θ", u"th": u"θ",
    u"CH": u"Χ", u"Ch": u"Χ", u"ch": u"χ",
    u"PS": u"Ψ", u"Ps": u"Ψ", u"ps": u"ψ",
})



# -*- coding: utf-8 -*-
"""
Belarusian (by) transliteration mappings.

ISO 639-1 language code: by

Transliteration follows ISO 9:1995 and BGN/PCGN romanization standards.
https://en.wikipedia.org/wiki/Belarusian_alphabet
https://en.wikipedia.org/wiki/Romanization_of_Belarusian
"""

import copy
from .ru import RU_CYR_TO_LAT_DICT

# Transliterate from Belarusian (based on ISO 9:1995 and BGN/PCGN)
BY_CYR_TO_LAT_DICT = copy.deepcopy(RU_CYR_TO_LAT_DICT)
# Change mapping to match Belarusian scientific transliteration
BY_CYR_TO_LAT_DICT[u"Г"] = u"H"
BY_CYR_TO_LAT_DICT[u"г"] = u"h"
BY_CYR_TO_LAT_DICT[u"Ё"] = u"Ë"
BY_CYR_TO_LAT_DICT[u"ё"] = u"ë"
BY_CYR_TO_LAT_DICT[u"Ж"] = u"Ž"
BY_CYR_TO_LAT_DICT[u"ж"] = u"ž"
BY_CYR_TO_LAT_DICT[u"Х"] = u"X"
BY_CYR_TO_LAT_DICT[u"х"] = u"x"
BY_CYR_TO_LAT_DICT[u"Ц"] = u"C"
BY_CYR_TO_LAT_DICT[u"ц"] = u"c"
BY_CYR_TO_LAT_DICT[u"Ч"] = u"Č"
BY_CYR_TO_LAT_DICT[u"ч"] = u"č"
BY_CYR_TO_LAT_DICT[u"Ш"] = u"Š"
BY_CYR_TO_LAT_DICT[u"ш"] = u"š"
BY_CYR_TO_LAT_DICT[u"Ы"] = u"Y"
BY_CYR_TO_LAT_DICT[u"ы"] = u"y"
BY_CYR_TO_LAT_DICT[u"Ь"] = u"'"
BY_CYR_TO_LAT_DICT[u"ь"] = u"'"
BY_CYR_TO_LAT_DICT[u"Э"] = u"Ė"
BY_CYR_TO_LAT_DICT[u"э"] = u"ė"
BY_CYR_TO_LAT_DICT[u"Ю"] = u"Ju"
BY_CYR_TO_LAT_DICT[u"ю"] = u"ju"
BY_CYR_TO_LAT_DICT[u"Я"] = u"Ja"
BY_CYR_TO_LAT_DICT[u"я"] = u"ja"
# Delete letters not used in Belarusian
del BY_CYR_TO_LAT_DICT[u"Щ"]
del BY_CYR_TO_LAT_DICT[u"щ"]
del BY_CYR_TO_LAT_DICT[u"Ъ"]
del BY_CYR_TO_LAT_DICT[u"ъ"]
# Update for Belarusian-specific letters
BY_CYR_TO_LAT_DICT.update({
    u"І": u"I", u"і": u"i",
    u"Ў": u"Ŭ", u"ў": u"ŭ"
})

# Latin to Cyrillic
BY_LAT_TO_CYR_DICT = {y: x for x, y in iter(BY_CYR_TO_LAT_DICT.items())}
BY_LAT_TO_CYR_DICT.update({
    u"JU": u"Ю", u"Ju": u"Ю", u"ju": u"ю",
    u"JA": u"Я", u"Ja": u"Я", u"ja": u"я",
    u"''": u"Ьь"  # Two apostrophes for Ьь
})
# Single apostrophe defaults to lowercase ь
BY_LAT_TO_CYR_DICT[u"'"] = u"ь"



# -*- coding: utf-8 -*-
"""
Ukrainian (ua) transliteration mappings.

ISO 639-1 language code: ua

Transliteration follows Scientific Ukrainian transliteration system.
"""

import copy
from .ru import RU_CYR_TO_LAT_DICT

# Transliterate from Ukrainian
UA_CYR_TO_LAT_DICT = copy.deepcopy(RU_CYR_TO_LAT_DICT)
# Change mapping to match with Scientific Ukrainian
UA_CYR_TO_LAT_DICT[u"Г"] = u"H"
UA_CYR_TO_LAT_DICT[u"г"] = u"h"
UA_CYR_TO_LAT_DICT[u"Ж"] = u"Ž"
UA_CYR_TO_LAT_DICT[u"ж"] = u"ž"
UA_CYR_TO_LAT_DICT[u"И"] = u"Y"
UA_CYR_TO_LAT_DICT[u"и"] = u"y"
UA_CYR_TO_LAT_DICT[u"Х"] = u"X"
UA_CYR_TO_LAT_DICT[u"х"] = u"x"
UA_CYR_TO_LAT_DICT[u"Ц"] = u"C"
UA_CYR_TO_LAT_DICT[u"ц"] = u"c"
UA_CYR_TO_LAT_DICT[u"Ч"] = u"Č"
UA_CYR_TO_LAT_DICT[u"ч"] = u"č"
UA_CYR_TO_LAT_DICT[u"Ш"] = u"Š"
UA_CYR_TO_LAT_DICT[u"ш"] = u"š"
UA_CYR_TO_LAT_DICT[u"Щ"] = u"Šč"
UA_CYR_TO_LAT_DICT[u"щ"] = u"šč"
UA_CYR_TO_LAT_DICT[u"Ю"] = u"Ju"
UA_CYR_TO_LAT_DICT[u"ю"] = u"ju"
UA_CYR_TO_LAT_DICT[u"Я"] = u"Ja"
UA_CYR_TO_LAT_DICT[u"я"] = u"ja"
# Delete unused letters
del UA_CYR_TO_LAT_DICT[u"Ё"]
del UA_CYR_TO_LAT_DICT[u"ё"]
del UA_CYR_TO_LAT_DICT[u"Ъ"]
del UA_CYR_TO_LAT_DICT[u"ъ"]
del UA_CYR_TO_LAT_DICT[u"Ы"]
del UA_CYR_TO_LAT_DICT[u"ы"]
del UA_CYR_TO_LAT_DICT[u"Э"]
del UA_CYR_TO_LAT_DICT[u"э"]

# Update for Ukrainian letters
UA_CYR_TO_LAT_DICT.update({
    u"Ґ": u"G", u"ґ": u"g",
    u"Є": u"Je", u"є": u"je",
    u"І": u"I", u"і": u"i",
    u"Ї": u"Ji", u"ї": u"ji"
})

# Latin to Cyrillic
UA_LAT_TO_CYR_DICT = {y: x for x, y in iter(UA_CYR_TO_LAT_DICT.items())}
UA_LAT_TO_CYR_DICT.update({
    u"JE": u"Є", u"jE": u"є",
    u"JI": u"Ї", u"jI": u"ї"
})



# -*- coding: utf-8 -*-
"""
Montenegrin (me) transliteration mappings.

ISO 3166-1 country code: me

Montenegrin Latin is based on Serbo-Croatian Latin, with the addition of the two letters Ś and Ź,
to replace the digraphs SJ and ZJ. These parallel the two letters of the Montenegrin Cyrillic
alphabet not found in Serbian, С́ and З́. These, respectively, could also be represented in the
original alphabets as šj and žj, and шj and жj.

Source: https://en.wikipedia.org/wiki/Montenegrin_alphabet#Latin_alphabet
Also see: http://news.bbc.co.uk/2/hi/8520466.stm
"""

import copy
from .sr import SR_CYR_TO_LAT_DICT

ME_CYR_TO_LAT_DICT = copy.deepcopy(SR_CYR_TO_LAT_DICT)
ME_CYR_TO_LAT_DICT.update({
    u'С́': u'Ś', u'с́': u'ś',  # Montenegrin
    u'З́': u'Ź', u'з́': u'ź'  # Montenegrin
})

# This dictionary is to transliterate from Montenegrin Latin to Cyrillic.
ME_LAT_TO_CYR_DICT = {y: x for x, y in iter(ME_CYR_TO_LAT_DICT.items())}
